/*
** ASS I/O v0.0. See readme.txt
** Copyright (C) 1998, Justin Frankel/Nullsoft Inc.
*/
#include <stdio.h>
#include "assio.h"

static ass_table_entry *ass_table;
static int num_assfiles;
static FILE *assfp;
static int lastpos;

int assinit(char *assfilename)
{
  char sig[] = "Nullsoft Ass v0.1", sig2[]="ass", sig3[]="Packed ass";
  char sigdata[64];
  if (assfp || ass_table) assquit();
  assfp=fopen(assfilename,"rb");
  if (!assfp) return 1;
  fread(sigdata,strlen(sig),1,assfp);
  if (memcmp(sigdata,sig,strlen(sig)))
  {
    fclose(assfp);
    assfp=NULL;
    return 2;
  }
  {
    int entry,of=0;
    fread(&num_assfiles,sizeof(int),1,assfp);
    ass_table =(ass_table_entry *) malloc(sizeof(ass_table_entry)*num_assfiles);
    if (!ass_table) 
    {
      fclose(assfp);
      assfp=NULL;
      return 3;
    }
    for (entry=0;entry<num_assfiles;entry++)
    {
      fread(ass_table[entry].filename,256,1,assfp);
      fread(&ass_table[entry].length,sizeof(int),1,assfp);
      ass_table[entry].offset_in_file = of;
      of += ass_table[entry].length;
    }
    fread(sigdata,strlen(sig2),1,assfp);
    if (memcmp(sigdata,sig2,strlen(sig2)))
    {
      fclose(assfp);
      free(ass_table);
      assfp=NULL;
      ass_table=0;
      return 2;
    }
    of=ftell(assfp);
    for (entry=0;entry<num_assfiles;entry++)
    {
      ass_table[entry].offset_in_file += of;
    }
  }
  fseek(assfp,-strlen(sig3),SEEK_END);
  fread(sigdata,strlen(sig3),1,assfp);
  if (memcmp(sigdata,sig3,strlen(sig3)))
  {
    fclose(assfp);
    free(ass_table);
    assfp=NULL;
    ass_table=0;
    return 2;
  }
  lastpos=0;
  return 0;
}

void assquit()
{
  if (assfp) { fclose(assfp); assfp=NULL; }
  if (ass_table) { free(ass_table); ass_table=0; }
  num_assfiles=0;
}

ASSFILE *assfopen(char *fn, char *mode)
{
  ASSFILE a, *b;
  FILE *fp;
  a.position=0;
  fp = fopen(fn,"rb");
  if (fp)
  {
    a.real_file=fp;
    a.ate=0;
  } 
  else if (!num_assfiles)
  {
    return 0;
  } 
  else
  {
    ass_table_entry *ass=ass_table;
    int x;
    for (x = 0; x < num_assfiles; x ++)
    {
      if (!stricmp(ass->filename,fn))
      {
        break;
      } else ass++;
    }
    if (x == num_assfiles) 
	{
		return 0;
	}
    a.real_file=0;
    a.ate=ass;
  }
  
  b=(ASSFILE *) malloc(sizeof(ASSFILE));
  *b=a;
  return b;
}

int assfclose(ASSFILE *a)
{
  if (!a) return 1;
  if (a->real_file)
  {
    fclose(a->real_file);
  }
  free(a);
  return 0;
}

int assfread(void *data, int size, int num, ASSFILE *s)
{
  int ret;
  if (s->real_file)
    return fread(data,size,num,s->real_file);
  if (s->position >= s->ate->length) return -1;
  if (s->position+s->ate->offset_in_file != lastpos)
    fseek(assfp,s->position+s->ate->offset_in_file,SEEK_SET);
  if (size*num + s->position > s->ate->length) num=(s->ate->length-s->position)/size;
  ret=fread(data,size,num,assfp);
  lastpos=ftell(assfp);
  s->position=lastpos-s->ate->offset_in_file;
  return ret;
}

char *assfgets(char *data, int num, ASSFILE *s)
{
  char *ret;
  if (s->real_file)
    return fgets(data,num,s->real_file);
  if (s->position+s->ate->offset_in_file != lastpos)
    fseek(assfp,s->position+s->ate->offset_in_file,SEEK_SET);
  if (s->position >= s->ate->length) return 0;
  ret=fgets(data,num,assfp);
  lastpos=ftell(assfp);
  s->position=lastpos-s->ate->offset_in_file;
  return ret;
}

int assfgetc(ASSFILE *s)
{
  int ret;
  if (s->real_file)
    return fgetc(s->real_file);
  if (s->position+s->ate->offset_in_file != lastpos)
    fseek(assfp,s->position+s->ate->offset_in_file,SEEK_SET);
  if (s->position >= s->ate->length) return -1;
  ret=fgetc(assfp);
  lastpos=++s->position+s->ate->offset_in_file;
  return ret;
}

int assfeof(ASSFILE *s)
{
  if (s->real_file)
    return feof(s->real_file);
  if (s->position >= s->ate->length) return 1;
  else return 0;
}

int assfseek(ASSFILE *s, int offset, int mode)
{
  int o;
  if (s->real_file)
    return fseek(s->real_file,offset,mode);
  switch (mode)
  {
    case SEEK_SET: o=s->ate->offset_in_file+offset; break;
    case SEEK_CUR: o=s->ate->offset_in_file+s->position+offset; break;
    case SEEK_END: o=s->ate->offset_in_file+s->ate->length+offset; break;
  }
  return fseek(assfp,o,SEEK_SET);
}

int assrewind(ASSFILE *s)
{
  return assfseek(s,0,SEEK_SET);
}

int assftell(ASSFILE *s)
{
  if (s->real_file)
    return ftell(s->real_file);
  return s->position;
}
