/*
** ASS I/O v0.0. See readme.txt
** Copyright (C) 1998, Justin Frankel/Nullsoft Inc.
*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

void ass_init();
void ass_addfile(char *filename);
void ass_write(char *assfilename);

typedef struct {
  char filename[256];
  int length;
} ass_table_entry;

ass_table_entry ass_file_table[4096];
int ass_numfiles;


void ass_init()
{
  printf("Ass packer: initialized clean ass\n");
  ass_numfiles=0;
}

void ass_addfile(char *filename)
{
  FILE *fp=fopen(filename,"rb");
  strncpy(ass_file_table[ass_numfiles].filename,filename,255);
  if (!fp)
  {
    printf("Error packing ass: can't find input file %s\n",filename);
    exit(1);
  }
  fseek(fp,0,SEEK_END);
  ass_file_table[ass_numfiles].length = ftell(fp);
  fclose(fp);
  printf("Ass packer: added file %s, length=%d\n",
    ass_file_table[ass_numfiles].filename,
    ass_file_table[ass_numfiles].length);
  ass_numfiles++;
}


void ass_write(char *assfilename)
{
  char sig[] = "Nullsoft Ass v0.1", sig2[]="ass", sig3[]="Packed ass";
  FILE *fp = fopen(assfilename,"wb");
  int tocb;
  if (!fp)
  { 
    printf("Error packing ass: output file not accessible\n");
    exit(1);
  }
  fwrite(sig,strlen(sig),1,fp);
  {
    int entry;
    fwrite(&ass_numfiles,sizeof(int),1,fp);
    for (entry=0;entry<ass_numfiles;entry++)
    {
      fwrite(&ass_file_table[entry].filename,256,1,fp);
      fwrite(&ass_file_table[entry].length,sizeof(int),1,fp);
    }
  }
  fwrite(sig2,strlen(sig2),1,fp);
  tocb=ftell(fp);
  {
    int entry;
    for (entry=0;entry<ass_numfiles;entry++)
    {
      FILE *in = fopen(ass_file_table[entry].filename,"rb");
      char *buf;
      if (!in) 
      {
        printf("Error packing ass: opening input file %s\n",
          ass_file_table[entry].filename);
        exit(1);
      }
      buf = (char *) malloc(ass_file_table[entry].length);
      if (!buf)
      {
        printf("Error packing ass: allocating %d bytes\n",
          ass_file_table[entry].length);
        exit(1);
      }
      fread(buf,ass_file_table[entry].length,1,in);
      fclose(in);
      fwrite(buf,ass_file_table[entry].length,1,fp);
      free(buf);
    }
  }
  fwrite(sig3,strlen(sig3),1,fp);
  printf("Ass packer: dirty ass successfully flushed to disk.\n"
         "TOC: %d, Total: %d\n",tocb,(int)ftell(fp));
  fclose(fp);
}

int main(int argc, char **argv)
{
  int c;
  if (argc < 3)
  {
    printf("Usage: packass assfile.ass myfile1.dat myfile2.dat ...\n");
    exit(1);
  }
  ass_init();
  for (c = 2; c < argc; c++) ass_addfile(argv[c]);
  ass_write(argv[1]);
  return 0;
}
